@@ -1,11 +1,12 @@
 from pyramid.compat import json
-import string
 import urllib
 
-from repoze.lru import LRUCache
+from plone.i18n.normalizer import urlnormalizer
+from pyramid.i18n import get_locale_name
 from pyramid.threadlocal import get_current_request
 from pyramid.url import resource_url
-from sqlalchemy.types import TypeDecorator, VARCHAR
+from repoze.lru import LRUCache
+from sqlalchemy.types import TypeDecorator, TEXT
 from sqlalchemy.ext.mutable import Mutable
 
 def dump_default(obj):
@@ -17,7 +18,7 @@ def dump_default(obj):
 class JsonType(TypeDecorator):
     """http://www.sqlalchemy.org/docs/core/types.html#marshal-json-strings
     """
-    impl = VARCHAR
+    impl = TEXT
 
     def process_bind_param(self, value, dialect):
         if value is not None:
@@ -208,23 +209,25 @@ def clear_cache(): # only useful for tests really
         setattr(request, _CACHE_ATTR, None)
     _lru_cache.clear()
 
-def extract_from_settings(prefix):
+def extract_from_settings(prefix, settings=None):
     """
-      >>> from pyramid.threadlocal import get_current_registry
-      >>> get_current_registry().settings = {
+      >>> settings = {
       ...     'kotti_twitter.foo_bar': '1', 'kotti.spam_eggs': '2'}
-      >>> print extract_from_settings('kotti_twitter.')
+      >>> print extract_from_settings('kotti_twitter.', settings)
       {'foo_bar': '1'}
     """
     from kotti import get_settings
+    settings = settings if settings is not None else get_settings()
     extracted = {}
-    for key, value in get_settings().items():
+    for key, value in settings.items():
         if key.startswith(prefix):
             extracted[key[len(prefix):]] = value
     return extracted
 
 def title_to_name(title):
-    okay = string.letters + string.digits + '-'
-    name = u'-'.join(title.lower().split())
-    name = u''.join(ch for ch in name if ch in okay)
-    return name
+    request = get_current_request()
+    if request is not None:
+        locale_name = get_locale_name(request)
+    else:
+        locale_name = 'en'
+    return unicode(urlnormalizer.normalize(title, locale_name))
