@@ -8,7 +8,6 @@ from pyramid.httpexceptions import HTTPFound
 from pyramid.exceptions import Forbidden
 import colander
 import deform
-from deform import Form
 from deform import Button
 from deform.widget import AutocompleteInputWidget
 from deform.widget import CheckedPasswordWidget
@@ -27,7 +26,8 @@ from kotti.security import set_groups
 from kotti.views.site_setup import CONTROL_PANEL_LINKS
 from kotti.views.util import template_api
 from kotti.views.util import is_root
-from kotti.views.util import FormController
+from kotti.views.util import AddFormView
+from kotti.views.util import EditFormView
 
 def roles_form_handler(context, request, available_role_names, groups_lister):
     changed = []
@@ -62,7 +62,7 @@ def roles_form_handler(context, request, available_role_names, groups_lister):
         if changed:
             request.session.flash(u'Your changes have been saved.', 'success')
         else:
-            request.session.flash(u'No changes made.', 'notice')
+            request.session.flash(u'No changes made.', 'info')
 
     return changed
 
@@ -88,7 +88,7 @@ def search_principals(request, context=None, ignore=None, extra=()):
             if p.name not in ignore:
                 entries.append((p, list_groups_ext(p.name, context)))
         if not found:
-            flash(u'No users or groups found.', 'notice')
+            flash(u'No users or groups found.', 'info')
 
     return entries
 
@@ -214,8 +214,8 @@ def user_schema(base=PrincipalFull()):
         has_password = False
     if has_password:
         schema['password'].description = (
-            u"Leave this empty and provide an email address below "
-            u"to send the user an email to set their own password."
+            u"Leave this empty and tick the 'Send password registration' box"
+            u"below to have the user set their own password."
             )
     schema['title'].title = u"Full name"
     return schema
@@ -254,6 +254,46 @@ def _massage_groups_out(appstruct):
     d['roles'] = roles
     return d
 
+class UserAddFormView(AddFormView):
+    buttons = (Button('add_user', u'Add User'), 'cancel')
+
+    def schema_factory(self):
+        schema = user_schema()
+        del schema['active']
+        schema.add(colander.SchemaNode(
+            colander.Boolean(),
+            name=u'send_email',
+            title=u'Send password registration link',
+            default=True,
+            ))
+        return schema
+
+    def add_user_success(self, appstruct):
+        appstruct.pop('csrf_token', None)
+        _massage_groups_in(appstruct)
+        name = appstruct['name'].lower()
+        send_email = appstruct.pop('send_email', False)
+        get_principals()[name] = appstruct
+        if send_email:
+            send_set_password(get_principals()[name], self.request)
+        self.request.session.flash(
+            u'%s added.' % appstruct['title'], 'success')
+        location = self.request.url.split('?')[0] + '?' + urlencode(
+            {'extra': name})
+        return HTTPFound(location=location)
+
+class GroupAddFormView(UserAddFormView):
+    buttons = (Button('add_group', u'Add Group'), 'cancel')
+
+    def schema_factory(self):
+        schema = group_schema()
+        del schema['active']
+        return schema
+
+    def add_group_success(self, appstruct):
+        appstruct['name'] = u'group:%s' % appstruct['name']
+        return self.add_user_success(appstruct)
+
 def users_manage(context, request):
     api = template_api(
         context, request,
@@ -286,61 +326,11 @@ def users_manage(context, request):
     available_roles = [ROLES[role_name] for role_name in USER_MANAGEMENT_ROLES]
 
     # Add forms:
-
-    # These are callbacks that we pass to the form controller.  They
-    # take control as soon as the data validates and are responsible
-    # for adding the actual principals and redirect:
-    def add_user(context, request, appstruct):
-        _massage_groups_in(appstruct)
-        name = appstruct['name'].lower()
-        send_email = appstruct.pop('send_email', False)
-        get_principals()[name] = appstruct
-        if send_email:
-            send_set_password(get_principals()[name], request)
-        request.session.flash(u'%s added.' % appstruct['title'], 'success')
-        location = request.url.split('?')[0] + '?' + urlencode({'extra': name})
-        return HTTPFound(location=location)
-        
-    def add_group(context, request, appstruct):
-        appstruct['name'] = u'group:%s' % appstruct['name']
-        return add_user(context, request, appstruct)
-
-    # The actual add forms:
-    uschema = user_schema()
-    del uschema['active']
-    uschema.add(colander.SchemaNode(
-        colander.Boolean(),
-        name=u'send_email',
-        title=u'Send password registration link',
-        default=True,
-        ))
-    user_form = Form(
-        uschema,
-        buttons=(Button('add-user', u'Add User'), 'cancel'),
-        )
-    user_fc = FormController(
-        user_form,
-        add=True,
-        add_item=add_user,
-        post_key='add-user',
-        )
-    user_addform = user_fc(context, request)
+    user_addform = UserAddFormView(context, request)()
     if request.is_response(user_addform):
         return user_addform
 
-    gschema = group_schema()
-    del gschema['active']
-    group_form = Form(
-        gschema,
-        buttons=(Button('add-group', u'Add Group'), 'cancel'),
-        )
-    group_fc = FormController(
-        group_form,
-        add=True,
-        add_item=add_group,
-        post_key='add-group',
-        )
-    group_addform = group_fc(context, request)
+    group_addform = GroupAddFormView(context, request)()
     if request.is_response(group_addform):
         return group_addform
 
@@ -348,10 +338,32 @@ def users_manage(context, request):
         'api': api,
         'entries': search_entries,
         'available_roles': available_roles,
-        'user_addform': user_addform,
-        'group_addform': group_addform,
+        'user_addform': user_addform['form'],
+        'group_addform': group_addform['form'],
         }
 
+class UserEditFormView(EditFormView):
+    @property
+    def success_url(self):
+        return self.request.url
+
+    def schema_factory(self):
+        return user_schema(PrincipalBasic())
+
+class UserManageFormView(UserEditFormView):
+    def schema_factory(self):
+        schema = user_schema()
+        del schema['name']
+        del schema['password']
+        return schema
+
+    def before(self, form):
+        form.appstruct = _massage_groups_out(self.context.__dict__.copy())
+
+    def save_success(self, appstruct):
+        _massage_groups_in(appstruct)
+        return super(UserEditFormView, self).save_success(appstruct)
+
 def user_manage(context, request):
     username = request.params['name']
     principal = get_principals()[username]
@@ -363,48 +375,27 @@ def user_manage(context, request):
         principal=principal,
         )
 
-    def edit_principal(context, request, appstruct):
-        _massage_groups_in(appstruct)
-        for key, value in appstruct.items():
-            setattr(context, key, value)
-        request.session.flash(u"Your changes have been saved.", 'success')
-        return HTTPFound(location=request.url)
-
-    uschema = user_schema()
-    del uschema['name']
-    del uschema['password']
-    user_form = Form(
-        uschema,
-        buttons=('save', 'cancel'),
-        )
-    user_fc = FormController(
-        user_form,
-        appstruct=lambda p: _massage_groups_out(p.__dict__.copy()),
-        edit_item=edit_principal,
-        )
-    form = user_fc(principal, request)
+    form = UserManageFormView(principal, request)()
     if request.is_response(form):
         return form
 
     return {
         'api': api,
-        'form': form,
+        'form': form['form'],
         }
 
 def preferences(context, request):
     api = template_api(context, request)
     api.page_title=u"My preferences - %s" % api.site_title
     user = api.user
-    uschema = user_schema(PrincipalBasic())
-    user_form = Form(uschema, buttons=('save', 'cancel'))
-    user_fc = FormController(user_form)
-    form = user_fc(user, request)
+
+    form = UserEditFormView(user, request)()
     if request.is_response(form):
         return form
 
     return {
         'api': api,
-        'form': form,
+        'form': form['form'],
         }
 
 def includeme(config):
